<?xml version="1.0"?>

<!--
xmi2svg 0.2

Copyright (c) 2004, STZ-IDA and PTV AG, Karlsruhe, Germany

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
-->

<!--
  -  TODO: * attribute sets for things like namespaces (get rid of c&p "italic")
  -->
<xsl:stylesheet version="1.0"
                xmlns="http://www.w3.org/2000/svg"
                xmlns:UML="org.omg.xmi.namespace.UML"
                xmlns:xlink="http://www.w3.org/1999/xlink"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

    <xsl:output media-type="image/svg+xml" method="xml" indent="yes"
                xmlns:xalan="http://xml.apache.org/xalan" xalan:indent-amount="2"/>
    
    <xsl:param name="diagramName" select="''"/>
    <xsl:param name="maxDepth" select="20"/>
    <!--xsl:param name="fontSize" select="11"/-->
    <xsl:param name="border" select="20"/>
    <xsl:variable name="commentBorder" select="10"/>
    <xsl:variable name="defaultFontSize" select="11"/>
    <xsl:variable name="defaultFontFamily" select="'Helvetica'"/>

    
    <!-- key for quick access to idref'd nodes -->
    <xsl:key name="xmi.id" match="UML:*" use="@xmi.id"/>


    <!-- some attribute sets for styling -->
    <xsl:attribute-set name="arrows">
        <!--xsl:attribute name="stroke-linejoin">bevel</xsl:attribute-->
        <xsl:attribute name="orient">auto</xsl:attribute>
    </xsl:attribute-set>

    
    <!--
      -  Generate markers for a specific color (the SVG
      -  spec lacks a way to inherit stroke attributes
      -  in a marker from the path where it's applied)
      -->
    <xsl:template name="create-color-id">
        <xsl:param name="color"/>
        <xsl:choose>
            <xsl:when test="$color='#000000'">black</xsl:when>
            <xsl:when test="starts-with($color, '#')">
                <xsl:value-of select="substring-after($color, '#')"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$color"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template name="create-markers">
        <xsl:param name="color"/>
        <xsl:variable name="color-id">
            <xsl:call-template name="create-color-id">
                <xsl:with-param name="color" select="$color"/>
            </xsl:call-template>
        </xsl:variable>
        <marker id="{concat('arrow-start-', $color-id)}" viewBox="0 0 20 20" refX="4.35" refY="10"
                markerWidth="20" markerHeight="20" fill="none" stroke="{$color}"
                xsl:use-attribute-sets="arrows">
            <path d="M 15 5 L 5 10 L 15 15"/>
        </marker>
        <marker id="{concat('arrow-end-', $color-id)}" viewBox="0 0 20 20" refX="15.65" refY="10"
                markerWidth="20" markerHeight="20" fill="none" stroke="{$color}"
                xsl:use-attribute-sets="arrows">
            <path d="M 5 5 L 15 10 L 5 15"/>
        </marker>
        <marker id="{concat('aggregation-composite-start-', $color-id)}" viewBox="0 0 30 20" refX="4.35" refY="10"
                markerWidth="30" markerHeight="20" fill="black" stroke="{$color}"
                xsl:use-attribute-sets="arrows">
            <path d="M 5 10 L 15 5 L 25 10 L 15 15 z"/>
        </marker>
        <marker id="{concat('aggregation-composite-end-', $color-id)}" viewBox="0 0 30 20" refX="25.65" refY="10"
                markerWidth="30" markerHeight="20" fill="black" stroke="{$color}"
                xsl:use-attribute-sets="arrows">
            <path d="M 5 10 L 15 5 L 25 10 L 15 15 z"/>
        </marker>
        <marker id="{concat('aggregation-composite-arrow-start-', $color-id)}" viewBox="0 0 40 20" refX="4.35" refY="10"
                markerWidth="40" markerHeight="20" stroke="{$color}"
                xsl:use-attribute-sets="arrows">
            <path d="M 5 10 L 15 5 L 25 10 L 15 15 z" fill="black"/>
            <path d="M 35 5 L 25 10 L 35 15" fill="none"/>
        </marker>
        <marker id="{concat('aggregation-composite-arrow-end-', $color-id)}" viewBox="0 0 40 20" refX="35.65" refY="10"
                markerWidth="40" markerHeight="20" stroke="{$color}"
                xsl:use-attribute-sets="arrows">
            <path d="M 15 10 L 25 5 L 35 10 L 25 15 z" fill="black"/>
            <path d="M 5 5 L 15 10 L 5 15" fill="none"/>
        </marker>
        <marker id="{concat('aggregation-aggregate-start-', $color-id)}" viewBox="0 0 30 20" refX="4.35" refY="10"
                markerWidth="30" markerHeight="20" fill="white" stroke="{$color}"
                xsl:use-attribute-sets="arrows">
            <path d="M 5 10 L 15 5 L 25 10 L 15 15 z"/>
        </marker>
        <marker id="{concat('aggregation-aggregate-end-', $color-id)}" viewBox="0 0 30 20" refX="25.65" refY="10"
                markerWidth="30" markerHeight="20" fill="white" stroke="{$color}"
                xsl:use-attribute-sets="arrows">
            <path d="M 5 10 L 15 5 L 25 10 L 15 15 z"/>
        </marker>
        <marker id="{concat('aggregation-aggregate-arrow-start-', $color-id)}" viewBox="0 0 40 20" refX="4.35" refY="10"
                markerWidth="40" markerHeight="20" stroke="{$color}"
                xsl:use-attribute-sets="arrows">
            <path d="M 5 10 L 15 5 L 25 10 L 15 15 z" fill="white"/>
            <path d="M 35 5 L 25 10 L 35 15" fill="none"/>
        </marker>
        <marker id="{concat('aggregation-aggregate-arrow-end-', $color-id)}" viewBox="0 0 40 20" refX="35.65" refY="10"
                markerWidth="40" markerHeight="20" stroke="{$color}"
                xsl:use-attribute-sets="arrows">
            <path d="M 15 10 L 25 5 L 35 10 L 25 15 z" fill="white"/>
            <path d="M 5 5 L 15 10 L 5 15" fill="none"/>
        </marker>
        <marker id="{concat('generalization-', $color-id)}" viewBox="0 0 20 20" refX="15.65" refY="10"
                markerWidth="20" markerHeight="20" fill="white" stroke="{$color}"
                xsl:use-attribute-sets="arrows">
            <path d="M 3 4 L 15 10 L 3 16 z"/>
        </marker>
    </xsl:template>

    
    <!--
      -  Create the desired diagram.
      -->
    <xsl:template match="UML:Diagram">
        <svg font-family="{$defaultFontFamily}" font-size="{$defaultFontSize}">
            <xsl:variable name="minX_1">
                <xsl:for-each select="UML:GraphElement.contained/UML:GraphNode[@isVisible='true']/
                                      UML:GraphElement.position/XMI.field[1]">
                    <xsl:sort select="." data-type="number" order="ascending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select="."/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="minY_1">
                <xsl:for-each select="UML:GraphElement.contained/UML:GraphNode[@isVisible='true']/
                                      UML:GraphElement.position/XMI.field[2]">
                    <xsl:sort select="." data-type="number" order="ascending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select="."/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="minX_2">
                <xsl:for-each select="UML:GraphElement.contained/UML:GraphEdge[@isVisible='true']/
                                      UML:GraphEdge.waypoints/XMI.field[position() mod 3 = 1]/XMI.field[1]">
                    <xsl:sort select=". + ../../../UML:GraphElement.position/XMI.field[1]" data-type="number" order="ascending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select=". + ../../../UML:GraphElement.position/XMI.field[1]"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="minY_2">
                <xsl:for-each select="UML:GraphElement.contained/UML:GraphEdge[@isVisible='true']/
                                      UML:GraphEdge.waypoints/XMI.field[position() mod 3 = 1]/XMI.field[2]">
                    <xsl:sort select=". + ../../../UML:GraphElement.position/XMI.field[2]" data-type="number" order="ascending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select=". + ../../../UML:GraphElement.position/XMI.field[2]"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="minX_3">
                <xsl:for-each select="UML:GraphElement.contained/UML:GraphEdge[@isVisible='true']/
                                      UML:GraphElement.contained/UML:GraphNode[@isVisible='true']/
                                      UML:GraphElement.contained/UML:GraphNode[@isVisible='true']">
                    <xsl:sort select="UML:GraphElement.position/XMI.field[1] +
                                      ../../UML:GraphElement.position/XMI.field[1] +
                                      ../../../../UML:GraphElement.position/XMI.field[1]"
                              data-type="number" order="ascending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select="UML:GraphElement.position/XMI.field[1] +
                                             ../../UML:GraphElement.position/XMI.field[1] +
                                             ../../../../UML:GraphElement.position/XMI.field[1]"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="minY_3">
                <xsl:for-each select="UML:GraphElement.contained/UML:GraphEdge[@isVisible='true']/
                                      UML:GraphElement.contained/UML:GraphNode[@isVisible='true']/
                                      UML:GraphElement.contained/UML:GraphNode[@isVisible='true']">
                    <xsl:sort select="UML:GraphElement.position/XMI.field[2] +
                                      ../../UML:GraphElement.position/XMI.field[2] +
                                      ../../../../UML:GraphElement.position/XMI.field[2]"
                              data-type="number" order="ascending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select="UML:GraphElement.position/XMI.field[2] +
                                             ../../UML:GraphElement.position/XMI.field[2] +
                                             ../../../../UML:GraphElement.position/XMI.field[2]"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="minX_4">
                <xsl:for-each select="UML:GraphElement.contained/UML:Polyline[@isVisible='true']/
                                      UML:Polyline.waypoints/XMI.field[position() mod 3 = 1]/XMI.field[1]">
                    <xsl:sort select="." data-type="number" order="ascending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select="."/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="minY_4">
                <xsl:for-each select="UML:GraphElement.contained/UML:Polyline[@isVisible='true']/
                                      UML:Polyline.waypoints/XMI.field[position() mod 3 = 1]/XMI.field[2]">
                    <xsl:sort select="." data-type="number" order="ascending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select="."/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="minX_5">
                <xsl:for-each select="UML:GraphElement.contained/UML:GraphNode[@isVisible='true']/
                                      UML:GraphElement.contained/UML:GraphNode[@isVisible='true']">
                    <xsl:sort select="UML:GraphElement.position/XMI.field[1] +
                                      ../../UML:GraphElement.position/XMI.field[1]"
                              data-type="number" order="ascending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select="UML:GraphElement.position/XMI.field[1] +
                                             ../../UML:GraphElement.position/XMI.field[1]"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="minY_5">
                <xsl:for-each select="UML:GraphElement.contained/UML:GraphNode[@isVisible='true']/
                                      UML:GraphElement.contained/UML:GraphNode[@isVisible='true']">
                    <xsl:sort select="UML:GraphElement.position/XMI.field[2] +
                                      ../../UML:GraphElement.position/XMI.field[2]"
                              data-type="number" order="ascending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select="UML:GraphElement.position/XMI.field[2] +
                                             ../../UML:GraphElement.position/XMI.field[2]"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="maxX_1">
                <xsl:for-each select="UML:GraphElement.contained/UML:GraphNode[@isVisible='true']">
                    <xsl:sort select="UML:GraphElement.position/XMI.field[1] +
                                      UML:GraphNode.size/XMI.field[1]" data-type="number" order="descending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select="UML:GraphElement.position/XMI.field[1] +
                                              UML:GraphNode.size/XMI.field[1]"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="maxY_1">
                <xsl:for-each select="UML:GraphElement.contained/UML:GraphNode[@isVisible='true']">
                    <xsl:sort select="UML:GraphElement.position/XMI.field[2] +
                                      UML:GraphNode.size/XMI.field[2]" data-type="number" order="descending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select="UML:GraphElement.position/XMI.field[2] +
                                              UML:GraphNode.size/XMI.field[2]"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="maxX_2">
                <xsl:for-each select="UML:GraphElement.contained/UML:GraphEdge[@isVisible='true']/
                                      UML:GraphEdge.waypoints/XMI.field[position() mod 3 = 1]/XMI.field[1]">
                    <xsl:sort select=". + ../../../UML:GraphElement.position/XMI.field[1]" data-type="number" order="descending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select=". + ../../../UML:GraphElement.position/XMI.field[1]"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="maxY_2">
                <xsl:for-each select="UML:GraphElement.contained/UML:GraphEdge[@isVisible='true']/
                                      UML:GraphEdge.waypoints/XMI.field[position() mod 3 = 1]/XMI.field[2]">
                    <xsl:sort select=". + ../../../UML:GraphElement.position/XMI.field[2]" data-type="number" order="descending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select=". + ../../../UML:GraphElement.position/XMI.field[2]"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="maxX_3">
                <xsl:for-each select="UML:GraphElement.contained/UML:GraphEdge[@isVisible='true']/
                                      UML:GraphElement.contained/UML:GraphNode[@isVisible='true']/
                                      UML:GraphElement.contained/UML:GraphNode[@isVisible='true']">
                    <xsl:sort select="UML:GraphElement.position/XMI.field[1] + UML:GraphNode.size/XMI.field[1] +
                                      ../../UML:GraphElement.position/XMI.field[1] +
                                      ../../../../UML:GraphElement.position/XMI.field[1]"
                              data-type="number" order="descending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select="UML:GraphElement.position/XMI.field[1] + UML:GraphNode.size/XMI.field[1] +
                                             ../../UML:GraphElement.position/XMI.field[1] +
                                             ../../../../UML:GraphElement.position/XMI.field[1]"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="maxY_3">
                <xsl:for-each select="UML:GraphElement.contained/UML:GraphEdge[@isVisible='true']/
                                      UML:GraphElement.contained/UML:GraphNode[@isVisible='true']/
                                      UML:GraphElement.contained/UML:GraphNode[@isVisible='true']">
                    <xsl:sort select="UML:GraphElement.position/XMI.field[2] + UML:GraphNode.size/XMI.field[2] +
                                      ../../UML:GraphElement.position/XMI.field[2] +
                                      ../../../../UML:GraphElement.position/XMI.field[2]"
                              data-type="number" order="descending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select="UML:GraphElement.position/XMI.field[2] + UML:GraphNode.size/XMI.field[2] +
                                             ../../UML:GraphElement.position/XMI.field[2] +
                                             ../../../../UML:GraphElement.position/XMI.field[2]"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="maxX_4">
                <xsl:for-each select="UML:GraphElement.contained/UML:Polyline[@isVisible='true']/
                                      UML:Polyline.waypoints/XMI.field[position() mod 3 = 1]/XMI.field[1]">
                    <xsl:sort select="." data-type="number" order="descending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select="."/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="maxY_4">
                <xsl:for-each select="UML:GraphElement.contained/UML:Polyline[@isVisible='true']/
                                      UML:Polyline.waypoints/XMI.field[position() mod 3 = 1]/XMI.field[2]">
                    <xsl:sort select="." data-type="number" order="descending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select="."/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="maxX_5">
                <xsl:for-each select="UML:GraphElement.contained/UML:GraphNode[@isVisible='true']/
                                      UML:GraphElement.contained/UML:GraphNode[@isVisible='true']">
                    <xsl:sort select="UML:GraphElement.position/XMI.field[1] + UML:GraphNode.size/XMI.field[1] +
                                      ../../UML:GraphElement.position/XMI.field[1]"
                              data-type="number" order="descending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select="UML:GraphElement.position/XMI.field[1] + UML:GraphNode.size/XMI.field[1] +
                                             ../../UML:GraphElement.position/XMI.field[1]"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <xsl:variable name="maxY_5">
                <xsl:for-each select="UML:GraphElement.contained/UML:GraphNode[@isVisible='true']/
                                      UML:GraphElement.contained/UML:GraphNode[@isVisible='true']">
                    <xsl:sort select="UML:GraphElement.position/XMI.field[2] + UML:GraphNode.size/XMI.field[2] +
                                      ../../UML:GraphElement.position/XMI.field[2]"
                              data-type="number" order="descending"/>
                    <xsl:if test="position() = 1">
                        <xsl:value-of select="UML:GraphElement.position/XMI.field[2] + UML:GraphNode.size/XMI.field[2] +
                                             ../../UML:GraphElement.position/XMI.field[2]"/>
                    </xsl:if>
                </xsl:for-each>
            </xsl:variable>
            <!--xsl:message>minX_1: <xsl:value-of select="$minX_1"/></xsl:message>
            <xsl:message>minY_1: <xsl:value-of select="$minY_1"/></xsl:message>
            <xsl:message>maxX_1: <xsl:value-of select="$maxX_1"/></xsl:message>
            <xsl:message>maxY_1: <xsl:value-of select="$maxY_1"/></xsl:message>
            <xsl:message>minX_2: <xsl:value-of select="$minX_2"/></xsl:message>
            <xsl:message>minY_2: <xsl:value-of select="$minY_2"/></xsl:message>
            <xsl:message>maxX_2: <xsl:value-of select="$maxX_2"/></xsl:message>
            <xsl:message>maxY_2: <xsl:value-of select="$maxY_2"/></xsl:message>
            <xsl:message>minX_3: <xsl:value-of select="$minX_3"/></xsl:message>
            <xsl:message>minY_3: <xsl:value-of select="$minY_3"/></xsl:message>
            <xsl:message>maxX_3: <xsl:value-of select="$maxX_3"/></xsl:message>
            <xsl:message>maxY_3: <xsl:value-of select="$maxY_3"/></xsl:message>
            <xsl:message>minX_4: <xsl:value-of select="$minX_4"/></xsl:message>
            <xsl:message>minY_4: <xsl:value-of select="$minY_4"/></xsl:message>
            <xsl:message>maxX_4: <xsl:value-of select="$maxX_4"/></xsl:message>
            <xsl:message>maxY_4: <xsl:value-of select="$maxY_4"/></xsl:message>
            <xsl:message>minX_5: <xsl:value-of select="$minX_5"/></xsl:message>
            <xsl:message>minY_5: <xsl:value-of select="$minY_5"/></xsl:message>
            <xsl:message>maxX_5: <xsl:value-of select="$maxX_5"/></xsl:message>
            <xsl:message>maxY_5: <xsl:value-of select="$maxY_5"/></xsl:message>
            <xsl:message>================</xsl:message-->
            <xsl:variable name="minX_1_2">
                <xsl:choose>
                    <xsl:when test="($minX_1 != '' and $minX_2 = '') or
                                    ($minX_1 != '' and $minX_2 != '' and $minX_1 &lt;= $minX_2)">
                        <xsl:value-of select="$minX_1"/>
                    </xsl:when>
                    <xsl:when test="($minX_1 = '' and $minX_2 != '') or
                                    ($minX_1 != '' and $minX_2 != '' and $minX_2 &lt;= $minX_1)">
                        <xsl:value-of select="$minX_2"/>
                    </xsl:when>
                    <!--xsl:otherwise>0</xsl:otherwise-->
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="minX_3_4">
                <xsl:choose>
                    <xsl:when test="($minX_3 != '' and $minX_4 = '') or
                                    ($minX_3 != '' and $minX_4 != '' and $minX_3 &lt;= $minX_4)">
                        <xsl:value-of select="$minX_3"/>
                    </xsl:when>
                    <xsl:when test="($minX_3 = '' and $minX_4 != '') or
                                    ($minX_3 != '' and $minX_4 != '' and $minX_4 &lt;= $minX_3)">
                        <xsl:value-of select="$minX_4"/>
                    </xsl:when>            
                    <!--xsl:otherwise>0</xsl:otherwise-->
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="minX_1_2_3_4">
                <xsl:choose>
                    <xsl:when test="($minX_1_2 != '' and $minX_3_4 = '') or
                                    ($minX_1_2 != '' and $minX_3_4 != '' and $minX_1_2 &lt;= $minX_3_4)">
                        <xsl:value-of select="$minX_1_2"/>
                    </xsl:when>
                    <xsl:when test="($minX_1_2 = '' and $minX_3_4 != '') or
                                    ($minX_1_2 != '' and $minX_3_4 != '' and $minX_3_4 &lt;= $minX_1_2)">
                        <xsl:value-of select="$minX_3_4"/>
                    </xsl:when>
                    <!--xsl:otherwise>0</xsl:otherwise-->
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="minX">
                <xsl:choose>
                    <xsl:when test="($minX_1_2_3_4 != '' and $minX_5 = '') or
                                    ($minX_1_2_3_4 != '' and $minX_5 != '' and $minX_1_2_3_4 &lt;= $minX_5)">
                        <xsl:value-of select="$minX_1_2_3_4"/>
                    </xsl:when>
                    <xsl:when test="($minX_1_2_3_4 = '' and $minX_5 != '') or
                                    ($minX_1_2_3_4 != '' and $minX_5 != '' and $minX_5 &lt;= $minX_1_2_3_4)">
                        <xsl:value-of select="$minX_5"/>
                    </xsl:when>
                    <xsl:otherwise>0</xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="maxX_1_2">
                <xsl:choose>
                    <xsl:when test="($maxX_1 != '' and $maxX_2 = '') or
                                    ($maxX_1 != '' and $maxX_2 != '' and $maxX_1 &gt;= $maxX_2)">
                        <xsl:value-of select="$maxX_1"/>
                    </xsl:when>
                    <xsl:when test="($maxX_1 = '' and $maxX_2 != '') or
                                    ($maxX_1 != '' and $maxX_2 != '' and $maxX_2 &gt;= $maxX_1)">
                        <xsl:value-of select="$maxX_2"/>
                    </xsl:when>
                    <!--xsl:otherwise>0</xsl:otherwise-->
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="maxX_3_4">
                <xsl:choose>
                    <xsl:when test="($maxX_3 != '' and $maxX_4 = '') or
                                    ($maxX_3 != '' and $maxX_4 != '' and $maxX_3 &gt;= $maxX_4)">
                        <xsl:value-of select="$maxX_3"/>
                    </xsl:when>
                    <xsl:when test="($maxX_3 = '' and $maxX_4 != '') or
                                    ($maxX_3 != '' and $maxX_4 != '' and $maxX_4 &gt;= $maxX_3)">
                        <xsl:value-of select="$maxX_4"/>
                    </xsl:when>
                    <!--xsl:otherwise>0</xsl:otherwise-->
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="maxX_1_2_3_4">
                <xsl:choose>
                    <xsl:when test="($maxX_1_2 != '' and $maxX_3_4 = '') or
                                    ($maxX_1_2 != '' and $maxX_3_4 != '' and $maxX_1_2 &gt;= $maxX_3_4)">
                        <xsl:value-of select="$maxX_1_2"/>
                    </xsl:when>
                    <xsl:when test="($maxX_1_2 = '' and $maxX_3_4 != '') or
                                    ($maxX_1_2 != '' and $maxX_3_4 != '' and $maxX_3_4 &gt;= $maxX_1_2)">
                        <xsl:value-of select="$maxX_3_4"/>
                    </xsl:when>
                    <!--xsl:otherwise>0</xsl:otherwise-->
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="maxX">
                <xsl:choose>
                    <xsl:when test="($maxX_1_2_3_4 != '' and $maxX_5 = '') or
                                    ($maxX_1_2_3_4 != '' and $maxX_5 != '' and $maxX_1_2_3_4 &gt;= $maxX_5)">
                        <xsl:value-of select="$maxX_1_2_3_4"/>
                    </xsl:when>
                    <xsl:when test="($maxX_1_2_3_4 = '' and $maxX_5 != '') or
                                    ($maxX_1_2_3_4 != '' and $maxX_5 != '' and $maxX_5 &gt;= $maxX_1_2_3_4)">
                        <xsl:value-of select="$maxX_5"/>
                    </xsl:when>
                    <xsl:otherwise>0</xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="minY_1_2">
                <xsl:choose>
                    <xsl:when test="($minY_1 != '' and $minY_2 = '') or
                                    ($minY_1 != '' and $minY_2 != '' and $minY_1 &lt;= $minY_2)">
                        <xsl:value-of select="$minY_1"/>
                    </xsl:when>
                    <xsl:when test="($minY_1 = '' and $minY_2 != '') or
                                    ($minY_1 != '' and $minY_2 != '' and $minY_2 &lt;= $minY_1)">
                        <xsl:value-of select="$minY_2"/>
                    </xsl:when>
                    <!--xsl:otherwise>0</xsl:otherwise-->
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="minY_3_4">
                <xsl:choose>
                    <xsl:when test="($minY_3 != '' and $minY_4 = '') or
                                    ($minY_3 != '' and $minY_4 != '' and $minY_3 &lt;= $minY_4)">
                        <xsl:value-of select="$minY_3"/>
                    </xsl:when>
                    <xsl:when test="($minY_3 = '' and $minY_4 != '') or
                                    ($minY_3 != '' and $minY_4 != '' and $minY_4 &lt;= $minY_3)">
                        <xsl:value-of select="$minY_4"/>
                    </xsl:when>
                    <!--xsl:otherwise>0</xsl:otherwise-->
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="minY_1_2_3_4">
                <xsl:choose>
                    <xsl:when test="($minY_1_2 != '' and $minY_3_4 = '') or
                                    ($minY_1_2 != '' and $minY_3_4 != '' and $minY_1_2 &lt;= $minY_3_4)">
                        <xsl:value-of select="$minY_1_2"/>
                    </xsl:when>
                    <xsl:when test="($minY_1_2 = '' and $minY_3_4 != '') or
                                    ($minY_1_2 != '' and $minY_3_4 != '' and $minY_3_4 &lt;= $minY_1_2)">
                        <xsl:value-of select="$minY_3_4"/>
                    </xsl:when>
                    <!--xsl:otherwise>0</xsl:otherwise-->
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="minY">
                <xsl:choose>
                    <xsl:when test="($minY_1_2_3_4 != '' and $minY_5 = '') or
                                    ($minY_1_2_3_4 != '' and $minY_5 != '' and $minY_1_2_3_4 &lt;= $minY_5)">
                        <xsl:value-of select="$minY_1_2_3_4"/>
                    </xsl:when>
                    <xsl:when test="($minY_1_2_3_4 = '' and $minY_5 != '') or
                                    ($minY_1_2_3_4 != '' and $minY_5 != '' and $minY_5 &lt;= $minY_1_2_3_4)">
                        <xsl:value-of select="$minY_5"/>
                    </xsl:when>
                    <xsl:otherwise>0</xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="maxY_1_2">
                <xsl:choose>
                    <xsl:when test="($maxY_1 != '' and $maxY_2 = '') or
                                    ($maxY_1 != '' and $maxY_2 != '' and $maxY_1 &gt;= $maxY_2)">
                        <xsl:value-of select="$maxY_1"/>
                    </xsl:when>
                    <xsl:when test="($maxY_1 = '' and $maxY_2 != '') or
                                    ($maxY_1 != '' and $maxY_2 != '' and $maxY_2 &gt;= $maxY_1)">
                        <xsl:value-of select="$maxY_2"/>
                    </xsl:when>
                    <!--xsl:otherwise>0</xsl:otherwise-->
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="maxY_3_4">
                <xsl:choose>
                    <xsl:when test="($maxY_3 != '' and $maxY_4 = '') or
                                    ($maxY_3 != '' and $maxY_4 != '' and $maxY_3 &gt;= $maxY_4)">
                        <xsl:value-of select="$maxY_3"/>
                    </xsl:when>
                    <xsl:when test="($maxY_3 = '' and $maxY_4 != '') or
                                    ($maxY_3 != '' and $maxY_4 != '' and $maxY_4 &gt;= $maxY_3)">
                        <xsl:value-of select="$maxY_4"/>
                    </xsl:when>
                    <!--xsl:otherwise>0</xsl:otherwise-->
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="maxY_1_2_3_4">
                <xsl:choose>
                    <xsl:when test="($maxY_1_2 != '' and $maxY_3_4 = '') or
                                    ($maxY_1_2 != '' and $maxY_3_4 != '' and $maxY_1_2 &gt;= $maxY_3_4)">
                        <xsl:value-of select="$maxY_1_2"/>
                    </xsl:when>
                    <xsl:when test="($maxY_1_2 = '' and $maxY_3_4 != '') or
                                    ($maxY_1_2 != '' and $maxY_3_4 != '' and $maxY_3_4 &gt;= $maxY_1_2)">
                        <xsl:value-of select="$maxY_3_4"/>
                    </xsl:when>
                    <!--xsl:otherwise>0</xsl:otherwise-->
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="maxY">
                <xsl:choose>
                    <xsl:when test="($maxY_1_2_3_4 != '' and $maxY_5 = '') or
                                    ($maxY_1_2_3_4 != '' and $maxY_5 != '' and $maxY_1_2_3_4 &gt;= $maxY_5)">
                        <xsl:value-of select="$maxY_1_2_3_4"/>
                    </xsl:when>
                    <xsl:when test="($maxY_1_2_3_4 = '' and $maxY_5 != '') or
                                    ($maxY_1_2_3_4 != '' and $maxY_5 != '' and $maxY_5 &gt;= $maxY_1_2_3_4)">
                        <xsl:value-of select="$maxY_5"/>
                    </xsl:when>
                    <xsl:otherwise>0</xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:message>minX: <xsl:value-of select="$minX"/></xsl:message>
            <xsl:message>minY: <xsl:value-of select="$minY"/></xsl:message>
            <xsl:message>maxX: <xsl:value-of select="$maxX"/></xsl:message>
            <xsl:message>maxY: <xsl:value-of select="$maxY"/></xsl:message>
            <xsl:variable name="shiftX">
                <xsl:value-of select="$border - $minX"/>
            </xsl:variable>
            <xsl:variable name="shiftY">
                <xsl:value-of select="$border - $minY"/>
            </xsl:variable>
            <xsl:variable name="width">
                <xsl:value-of select="$maxX - $minX + $border*2"/>
            </xsl:variable>
            <xsl:variable name="height">
                <xsl:value-of select="$maxY - $minY + $border*2"/>
            </xsl:variable>
            <xsl:attribute name="width">
                <xsl:value-of select="$width"/>
            </xsl:attribute>
            <xsl:attribute name="height">
                <xsl:value-of select="$height"/>
            </xsl:attribute>
            <defs>
                <xsl:call-template name="create-markers">
                    <xsl:with-param name="color" select="'black'"/>
                </xsl:call-template>
                <xsl:variable name="diagramId">
                    <xsl:value-of select="@xmi.id"/>
                </xsl:variable>
                <xsl:for-each select="//UML:Diagram[@xmi.id=$diagramId]//
                                        UML:Property[@key='stroke' and @value != 'black' and @value != '#000000']">
                    <xsl:sort select="@value" data-type="text" order="ascending"/>
                    <xsl:variable name="index" select="position()"/>
                    <xsl:variable name="color">
                        <xsl:value-of select="@value"/>
                    </xsl:variable>
                    <xsl:variable name="omit">
                        <xsl:for-each select="//UML:Diagram[@xmi.id=$diagramId]//
                                                UML:Property[@key='stroke' and @value != 'black' and @value != '#000000']">
                            <xsl:sort select="@value" data-type="text" order="ascending"/>
                            <xsl:if test="position() > $index and @value=$color">true</xsl:if>
                            <!-- note that "true" may be output a number of times,
                                 but that doesn't matter -->
                        </xsl:for-each>
                    </xsl:variable>
                    <xsl:if test="$omit = ''">
                        <xsl:call-template name="create-markers">
                            <xsl:with-param name="color" select="$color"/>
                        </xsl:call-template>
                    </xsl:if>
                </xsl:for-each>
            </defs>
            <!-- fill = #ffffe3 -->
            <rect x="0" y="0" stroke="none" fill="#ffffe3"
                  width="{$width}" height="{$height}"/>
            <g>
                <xsl:if test="$shiftX != 0 or $shiftY != 0">
                    <xsl:attribute name="transform">
                        <xsl:value-of select="concat('translate(', $shiftX, ',', $shiftY, ')')"/>
                    </xsl:attribute>
                </xsl:if>
                <xsl:call-template name="processGraphNode">
                    <xsl:with-param name="depth" select="0"/>
                    <xsl:with-param name="fontSize">
                        <xsl:call-template name="getFontSize"/>
                    </xsl:with-param>
                    <xsl:with-param name="fontColor">
                        <xsl:call-template name="getFontColor"/>
                    </xsl:with-param>
                    <xsl:with-param name="fontFamily">
                        <xsl:call-template name="getFontFamily"/>
                    </xsl:with-param>
                </xsl:call-template>
            </g>
        </svg>
    </xsl:template>

    
    <!--
      -  Determine the font properties for a graph node.
      -->
    <xsl:template name="getFontSize">
        <xsl:choose>
            <xsl:when test="UML:DiagramElement.property/UML:Property[@key='font-size']">
                <xsl:value-of select="UML:DiagramElement.property/UML:Property[@key='font-size']/@value"/>
            </xsl:when>
        </xsl:choose>
    </xsl:template>
    <xsl:template name="getFontColor">
        <xsl:choose>
            <xsl:when test="UML:DiagramElement.property/UML:Property[@key='font-color']">
                <xsl:value-of select="UML:DiagramElement.property/UML:Property[@key='font-color']/@value"/>
            </xsl:when>
        </xsl:choose>
    </xsl:template>
    <xsl:template name="getFontFamily">
        <xsl:choose>
            <xsl:when test="UML:DiagramElement.property/UML:Property[@key='font-family']">
                <xsl:value-of select="UML:DiagramElement.property/UML:Property[@key='font-family']/@value"/>
            </xsl:when>
        </xsl:choose>
    </xsl:template>

    
    <!--
      -  Get the marker for an AssociationEnd.
      -->
    <xsl:template name="getMarker">
        <xsl:choose>
            <xsl:when test="@aggregation='composite'">aggregation-composite</xsl:when>
            <xsl:when test="@aggregation='aggregate'">aggregation-aggregate</xsl:when>
        </xsl:choose>
    </xsl:template>

    
    <!--
      -  Template for determining the various style attributes.
      -->
    <xsl:template name="getFillStyle">
        <xsl:choose>
            <xsl:when test="(UML:Polyline.waypoints and @closed='true') or UML:Ellipse.center or
                            UML:GraphElement.contained/UML:GraphNode/UML:GraphElement.semanticModel/
                             UML:SimpleSemanticModelElement/@typeInfo='CompartmentSeparator' or
                            UML:GraphElement.semanticModel//UML:Actor or
                            UML:GraphElement.semanticModel//UML:UseCase or
                            UML:GraphElement.semanticModel//UML:Collaboration or
                            UML:GraphElement.semanticModel//UML:ActionState or
                            UML:GraphElement.semanticModel/*[@presentation='Ball']//UML:Interface">
                <xsl:choose>
                    <xsl:when test="UML:DiagramElement.property/UML:Property[@key='fill']">
                        <xsl:value-of select="UML:DiagramElement.property/UML:Property[@key='fill']/@value"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:text>white</xsl:text>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="(UML:GraphElement.semanticModel/UML:SimpleSemanticModelElement[@typeInfo='NameCompartment'] and
                             ../../UML:GraphElement.semanticModel//UML:Package) or
                            (UML:GraphElement.semanticModel/UML:SimpleSemanticModelElement[@typeInfo='BodyCompartment'] and
                             ../../UML:GraphElement.semanticModel//UML:Package)">
                <xsl:choose>
                    <xsl:when test="../../UML:DiagramElement.property/UML:Property[@key='fill']">
                        <xsl:value-of select="../../UML:DiagramElement.property/UML:Property[@key='fill']/@value"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:text>white</xsl:text>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="UML:GraphElement.semanticModel//UML:Comment">
                <xsl:choose>
                    <xsl:when test="UML:DiagramElement.property/UML:Property[@key='fill']">
                        <xsl:value-of select="UML:DiagramElement.property/UML:Property[@key='fill']/@value"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:text>yellow</xsl:text>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="UML:GraphElement.semanticModel//UML:Pseudostate">
                <xsl:choose>
                    <xsl:when test="UML:DiagramElement.property/UML:Property[@key='fill']">
                        <xsl:value-of select="UML:DiagramElement.property/UML:Property[@key='fill']/@value"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:for-each select="key('xmi.id', UML:GraphElement.semanticModel//UML:Pseudostate/@xmi.idref)">
                            <xsl:choose>
                                <xsl:when test="@kind='choice' or
                                                @kind='junction' and
                                                ancestor::UML:Diagram/UML:GraphElement.semanticModel/
                                                UML:SimpleSemanticModelElement/@typeInfo != 'StateDiagram'">
                                    <xsl:text>white</xsl:text>
                                </xsl:when>
                                <xsl:when test="@kind='initial' or
                                                @kind='junction' or
                                                @kind='join' or
                                                @kind='fork'">black</xsl:when>
                                <xsl:otherwise>none</xsl:otherwise>
                            </xsl:choose>
                        </xsl:for-each>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="UML:GraphElement.semanticModel//UML:FinalState or
                            UML:GraphElement.semanticModel//UML:SynchState or
                            UML:GraphElement.semanticModel//UML:ObjectFlowState">
                <xsl:choose>
                    <xsl:when test="UML:DiagramElement.property/UML:Property[@key='fill']">
                        <xsl:value-of select="UML:DiagramElement.property/UML:Property[@key='fill']/@value"/>
                    </xsl:when>
                    <xsl:otherwise>white</xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>none</xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template name="getStrokeStyle">
        <xsl:choose>
            <xsl:when test="UML:DiagramElement.property/UML:Property[@key='stroke']">
                <xsl:value-of select="UML:DiagramElement.property/UML:Property[@key='stroke']/@value"/>
            </xsl:when>
            <xsl:when test="UML:GraphElement.contained/UML:GraphNode/UML:GraphElement.semanticModel/
                            UML:SimpleSemanticModelElement[@typeInfo='CompartmentSeparator']">
                <xsl:text>black</xsl:text>
            </xsl:when>
            <xsl:when test="UML:GraphElement.semanticModel/UML:SimpleSemanticModelElement[@typeInfo='CompartmentSeparator'] or
                            UML:GraphElement.semanticModel/UML:SimpleSemanticModelElement[@typeInfo='RegionSeparator']">
                <xsl:text>black</xsl:text>
            </xsl:when>
            <xsl:when test="UML:GraphElement.semanticModel/UML:SimpleSemanticModelElement[@typeInfo='NameCompartment'] and
                            ../../UML:GraphElement.semanticModel//UML:Package">
                <xsl:text>black</xsl:text>
            </xsl:when>
            <xsl:when test="UML:GraphElement.semanticModel/UML:SimpleSemanticModelElement[@typeInfo='BodyCompartment'] and
                            ../../UML:GraphElement.semanticModel//UML:Package">
                <xsl:text>black</xsl:text>
            </xsl:when>
            <xsl:when test="UML:GraphElement.semanticModel//UML:Collaboration or
                            UML:GraphElement.semanticModel//UML:Actor or
                            UML:GraphElement.semanticModel//UML:UseCase or
                            UML:GraphElement.semanticModel//UML:Comment or
                            UML:GraphEdge.waypoints or UML:Polyline.waypoints or
                            UML:Ellipse.center or
                            UML:GraphElement.semanticModel/*[@presentation='Ball']//UML:Interface or
                            UML:GraphElement.semanticModel//UML:Pseudostate or
                            UML:GraphElement.semanticModel//UML:FinalState or
                            UML:GraphElement.semanticModel//UML:SynchState or
                            UML:GraphElement.semanticModel//UML:ObjectFlowState or
                            UML:GraphElement.semanticModel//UML:ActionState">
                <xsl:text>black</xsl:text>
            </xsl:when>
            <xsl:otherwise>none</xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="*" mode="style">
        <xsl:param name="ignore-stroke-and-fill" select="''"/>
        <xsl:if test="$ignore-stroke-and-fill != 'true'">
            <xsl:attribute name="stroke">
                <xsl:call-template name="getStrokeStyle"/>
            </xsl:attribute>
            <xsl:attribute name="fill">
                <xsl:call-template name="getFillStyle"/>
            </xsl:attribute>
        </xsl:if>
        <xsl:if test="UML:DiagramElement.property/UML:Property[@key='fill-opacity']">
            <xsl:attribute name="fill-opacity">
                <xsl:value-of select="UML:DiagramElement.property/UML:Property[@key='fill-opacity']/@value"/>
            </xsl:attribute>
        </xsl:if>
        <xsl:if test="UML:GraphElement.semanticModel//UML:Collaboration or
                      UML:GraphElement.semanticModel//UML:ClassifierRole or
                      UML:GraphElement.semanticModel//UML:Dependency or
                      UML:GraphElement.semanticModel//UML:Include or
                      UML:GraphElement.semanticModel//UML:Extend or
                      (UML:GraphElement.semanticModel//UML:Abstraction and
                       UML:GraphElement.contained/UML:GraphNode[@isVisible='true' and
                       .//UML:SimpleSemanticModelElement[@typeInfo='StereotypeCompartment']]) or
                      UML:GraphElement.semanticModel/UML:SimpleSemanticModelElement/@typeInfo = 'CommentLink' or
                      UML:GraphElement.semanticModel/UML:SimpleSemanticModelElement/@typeInfo = 'AssociationClassEdge' or
                      UML:GraphElement.semanticModel/UML:SimpleSemanticModelElement/@typeInfo = 'RegionSeparator'">
            <xsl:attribute name="stroke-dasharray">5,5</xsl:attribute>
        </xsl:if>
    </xsl:template>

    
    <!--
      -  Use this template to override graph node processing
      -  (e.g. for filtering). Call "doProcessGraphNode" to
      -  actuall process the node.
      -->
    <xsl:template name="processGraphNode">
        <xsl:param name="depth"/>
        <xsl:param name="fontSize"/>
        <xsl:param name="fontColor"/>
        <xsl:param name="fontFamily"/>

        <!-- default behaviour: simply pass the call through -->
        <xsl:call-template name="doProcessGraphNode">
            <xsl:with-param name="depth" select="$depth"/>
            <xsl:with-param name="fontSize" select="$fontSize"/>
            <xsl:with-param name="fontColor" select="$fontColor"/>
            <xsl:with-param name="fontFamily" select="$fontFamily"/>
        </xsl:call-template>
    </xsl:template>

    
    <!--
      -  Recursively iterate over graph nodes.
      -->
    <xsl:template name="doProcessGraphNode">
        <xsl:param name="depth"/>
        <xsl:param name="fontSize"/>
        <xsl:param name="fontColor"/>
        <xsl:param name="fontFamily"/>
        <xsl:variable name="depthVar">
            <xsl:value-of select="$depth + 1"/>
        </xsl:variable>
        <xsl:variable name="fontSizeVarTemp">
            <xsl:call-template name="getFontSize"/>
        </xsl:variable>
        <xsl:variable name="fontSizeVar">
            <xsl:choose>
                <xsl:when test="$fontSizeVarTemp=''">
                    <xsl:value-of select="$fontSize"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$fontSizeVarTemp"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="fontColorVarTemp">
            <xsl:call-template name="getFontColor"/>
        </xsl:variable>
        <xsl:variable name="fontColorVar">
            <xsl:choose>
                <xsl:when test="$fontColorVarTemp=''">
                    <xsl:value-of select="$fontColor"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$fontColorVarTemp"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="fontFamilyVarTemp">
            <xsl:call-template name="getFontFamily"/>
        </xsl:variable>
        <xsl:variable name="fontFamilyVar">
            <xsl:choose>
                <xsl:when test="$fontFamilyVarTemp=''">
                    <xsl:value-of select="$fontFamily"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$fontFamilyVarTemp"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <g>
            <xsl:if test="UML:GraphElement.position">
                <xsl:variable name="shiftX">
                    <xsl:value-of select="UML:GraphElement.position/XMI.field[1]"/>
                </xsl:variable>
                <xsl:variable name="shiftY">
                    <xsl:value-of select="UML:GraphElement.position/XMI.field[2]"/>
                </xsl:variable>
                <xsl:if test="$shiftX != 0 or $shiftY != 0">
                    <xsl:attribute name="transform">
                        <xsl:value-of select="concat('translate(', $shiftX, ',', $shiftY, ')')"/>
                    </xsl:attribute>
                </xsl:if>
            </xsl:if>

            <xsl:if test="((../../UML:GraphElement.semanticModel//UML:Operation or
                            ../../UML:GraphElement.semanticModel//UML:Attribute) and
                            key('xmi.id', ../../UML:GraphElement.semanticModel//@xmi.idref)/@ownerScope='classifier') or
                          ../../../../UML:GraphElement.semanticModel//UML:ObjectFlowState">
                <xsl:variable name="fontSize">
                    <xsl:choose>
                        <xsl:when test="$fontSizeVar=''">11</xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$fontSizeVar"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <line x1="0" y1="{UML:GraphNode.size/XMI.field[2]}"
                      x2="{UML:GraphNode.size/XMI.field[1]}"
                      y2="{UML:GraphNode.size/XMI.field[2]}">
                    <xsl:attribute name="stroke">
                        <xsl:choose>
                            <xsl:when test="$fontColorVar=''">black</xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select="$fontColorVar"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:attribute>
                </line>
            </xsl:if>
            
            <xsl:choose>
                <xsl:when test="UML:GraphElement.semanticModel//UML:ActionState">
                    <rect>
                        <xsl:apply-templates select="." mode="style"/>
                        <xsl:attribute name="rx">
                            <xsl:value-of select="UML:GraphNode.size/XMI.field[1] * 0.07"/>
                        </xsl:attribute>
                        <xsl:attribute name="ry">
                            <xsl:value-of select="UML:GraphNode.size/XMI.field[2] * 0.5"/>
                        </xsl:attribute>
                        <xsl:apply-templates select="*" mode="attrib"/>
                    </rect>
                </xsl:when>
                <xsl:when test="UML:Ellipse.center">
                    <ellipse cx="{UML:Ellipse.center/XMI.field[1]}"
                             cy="{UML:Ellipse.center/XMI.field[2]}"
                             rx="{@radiusX}" ry="{@radiusY}">
                        <xsl:apply-templates select="." mode="style"/>
                    </ellipse>
                </xsl:when>
                <xsl:when test="UML:GraphElement.semanticModel//UML:Pseudostate[key('xmi.id', @xmi.idref)/@kind='junction'] and
                                ancestor::UML:Diagram/UML:GraphElement.semanticModel/
                                UML:SimpleSemanticModelElement/@typeInfo != 'StateDiagram'">
                    <path>
                        <xsl:attribute name="d">
                            <xsl:text>M </xsl:text>
                            <xsl:value-of select="UML:GraphNode.size/XMI.field[1] div 2"/>
                            <xsl:text> 0 L </xsl:text>
                            <xsl:value-of select="UML:GraphNode.size/XMI.field[1]"/>
                            <xsl:text> </xsl:text>
                            <xsl:value-of select="UML:GraphNode.size/XMI.field[2] div 2"/>
                            <xsl:text> L </xsl:text>
                            <xsl:value-of select="UML:GraphNode.size/XMI.field[1] div 2"/>
                            <xsl:text> </xsl:text>
                            <xsl:value-of select="UML:GraphNode.size/XMI.field[2]"/>
                            <xsl:text> L 0 </xsl:text>
                            <xsl:value-of select="UML:GraphNode.size/XMI.field[2] div 2"/>
                            <xsl:text> z</xsl:text>
                        </xsl:attribute>
                        <xsl:apply-templates select="." mode="style"/>
                    </path>
                </xsl:when>
                <xsl:when test="UML:GraphElement.semanticModel//UML:Collaboration or
                                UML:GraphElement.semanticModel//UML:UseCase">
                    <xsl:variable name="rx" select="UML:GraphNode.size/XMI.field[1] div 2"/>
                    <xsl:variable name="ry" select="UML:GraphNode.size/XMI.field[2] div 2"/>
                    <ellipse cx="{$rx}" cy="{$ry}"
                             rx="{$rx}" ry="{$ry}">
                        <xsl:apply-templates select="." mode="style"/>
                    </ellipse>
                </xsl:when>
                <xsl:when test="UML:GraphElement.semanticModel//UML:SynchState">
                    <xsl:variable name="rx" select="UML:GraphNode.size/XMI.field[1] div 2"/>
                    <xsl:variable name="ry" select="UML:GraphNode.size/XMI.field[2] div 2"/>
                    <ellipse cx="{$rx}" cy="{$ry}"
                             rx="{$rx}" ry="{$ry}">
                        <xsl:apply-templates select="." mode="style"/>
                    </ellipse>
                    <text x="{$rx}" y="{$ry + 4}"
                          font-family="Helvetica" font-size="12" font-weight="bold"
                          text-anchor="middle">*</text>
                </xsl:when>
                <xsl:when test="UML:GraphElement.semanticModel//UML:Pseudostate and
                                key('xmi.id', UML:GraphElement.semanticModel//UML:Pseudostate/@xmi.idref)/
                                @kind != 'fork' and
                                key('xmi.id', UML:GraphElement.semanticModel//UML:Pseudostate/@xmi.idref)/
                                @kind != 'join'">
                    <xsl:variable name="rx" select="UML:GraphNode.size/XMI.field[1] div 2"/>
                    <xsl:variable name="ry" select="UML:GraphNode.size/XMI.field[2] div 2"/>
                    <ellipse cx="{$rx}" cy="{$ry}"
                             rx="{$rx}" ry="{$ry}">
                        <xsl:apply-templates select="." mode="style"/>
                    </ellipse>
                    <xsl:variable name="kind">
                        <xsl:value-of select="key('xmi.id', UML:GraphElement.semanticModel//
                                                            UML:Pseudostate/@xmi.idref)/@kind"/>
                    </xsl:variable>
                    <xsl:choose>
                        <xsl:when test="$kind = 'deepHistory'">
                            <text x="{$rx}" y="{$ry + 4}"
                                  font-family="Helvetica" font-size="12" font-weight="bold"
                                  text-anchor="middle">H*</text>
                        </xsl:when>
                        <xsl:when test="$kind = 'shallowHistory'">
                            <text x="{$rx}" y="{$ry + 4}"
                                  font-family="Helvetica" font-size="12" font-weight="bold"
                                  text-anchor="middle">H</text>
                        </xsl:when>
                    </xsl:choose>
                </xsl:when>
                <xsl:when test="UML:GraphElement.semanticModel//UML:FinalState">
                    <xsl:variable name="rx" select="UML:GraphNode.size/XMI.field[1] div 2"/>
                    <xsl:variable name="ry" select="UML:GraphNode.size/XMI.field[2] div 2"/>
                    <ellipse cx="{$rx}" cy="{$ry}"
                             rx="{$rx}" ry="{$ry}">
                        <xsl:apply-templates select="." mode="style"/>
                    </ellipse>
                    <ellipse cx="{$rx}" cy="{$ry}"
                             rx="{$rx * 0.7}" ry="{$ry * 0.7}"
                             stroke="black" fill="black"/>
                </xsl:when>
                <xsl:when test="UML:GraphElement.semanticModel/*[@presentation='Ball']//UML:Interface">
                    <xsl:variable name="rx" select="UML:GraphNode.size/XMI.field[1] div 2 * 2 div 3"/>
                    <xsl:variable name="ry" select="UML:GraphNode.size/XMI.field[2] div 2 * 2 div 3"/>
                    <ellipse cx="{$rx*1.5}"
                             cy="{$ry*1.5}"
                             rx="{$rx}" ry="{$ry}">
                        <xsl:apply-templates select="." mode="style"/>
                    </ellipse>
                </xsl:when>
                <xsl:when test="UML:GraphElement.semanticModel//UML:Actor and
                                not(UML:GraphElement.contained//UML:SimpleSemanticModelElement[@typeInfo='NameCompartment'])">
                    <xsl:variable name="width" select="UML:GraphNode.size/XMI.field[1]"/>
                    <xsl:variable name="height" select="UML:GraphNode.size/XMI.field[2]"/>
                    <line x1="0" y1="{$height}"
                          x2="{$width div 2}" y2="{2 div 3 * $height}">
                        <xsl:apply-templates select="." mode="style"/>
                    </line>
                    <line x1="{$width}" y1="{$height}"
                          x2="{$width div 2}" y2="{2 div 3 * $height}">
                        <xsl:apply-templates select="." mode="style"/>
                    </line>
                    <line x1="{$width div 2}" y1="{2 div 3 * $height}"
                          x2="{$width div 2}" y2="{1.5 div 6 * $height}">
                        <xsl:apply-templates select="." mode="style"/>
                    </line>
                    <line x1="0" y1="{1 div 3 * $height}"
                          x2="{$width}" y2="{1 div 3 * $height}">
                        <xsl:apply-templates select="." mode="style"/>
                    </line>
                    <ellipse cx="{$width div 2}" cy="{0.75 div 6 * $height}"
                             rx="{$width * 0.175}" ry="{0.75 div 6 * $height}">
                        <xsl:apply-templates select="." mode="style"/>
                    </ellipse>
                </xsl:when>
                <xsl:when test="UML:GraphEdge.waypoints or UML:Polyline.waypoints">
                    <xsl:variable name="commands">
                        <xsl:for-each select="UML:GraphEdge.waypoints/XMI.field|UML:Polyline.waypoints/XMI.field">
                            <xsl:variable name="pos">
                                <xsl:value-of select="count(preceding-sibling::XMI.field)"/>
                            </xsl:variable>
                            <!-- note: no curve support yet -->
                            <xsl:if test="$pos mod 3 = 0">
                                <xsl:choose>
                                    <xsl:when test="$pos = 0">M </xsl:when>
                                    <xsl:otherwise>L </xsl:otherwise>
                                </xsl:choose>
                                <xsl:value-of select="XMI.field[1]"/>
                                <xsl:text> </xsl:text>
                                <xsl:value-of select="XMI.field[2]"/>
                                <xsl:text> </xsl:text>
                            </xsl:if>
                        </xsl:for-each>
                        <xsl:if test="UML:Polyline.waypoints and @closed='true'">
                            <xsl:text>z</xsl:text>
                        </xsl:if>
                    </xsl:variable>

                    <!-- find the association ends -->
                    <xsl:variable name="start-id">
                        <xsl:value-of select="UML:GraphElement.contained/UML:GraphNode[1]//UML:AssociationEnd/@xmi.idref"/>
                    </xsl:variable>
                    <xsl:variable name="start-marker-temp">
                        <xsl:for-each select="key('xmi.id', $start-id)">
                            <xsl:call-template name="getMarker"/>
                        </xsl:for-each>
                    </xsl:variable>
                    <xsl:variable name="end-id">
                        <xsl:value-of select="UML:GraphElement.contained/UML:GraphNode[2]//UML:AssociationEnd/@xmi.idref"/>
                    </xsl:variable>
                    <xsl:variable name="end-marker-temp">
                        <xsl:for-each select="key('xmi.id', $end-id)">
                            <xsl:call-template name="getMarker"/>
                        </xsl:for-each>
                    </xsl:variable>
                    <xsl:variable name="start-marker">
                        <xsl:choose>
                            <xsl:when test="$start-marker-temp = ''">
                                <xsl:if test="key('xmi.id', $start-id)/@isNavigable='true' and
                                              key('xmi.id', $end-id)/@isNavigable='false'">
                                    <xsl:text>arrow</xsl:text>
                                </xsl:if>
                            </xsl:when>
                            <xsl:when test="$start-marker-temp = 'aggregation-composite' and
                                            key('xmi.id', $start-id)/@isNavigable='true' and
                                            key('xmi.id', $end-id)/@isNavigable='false'">
                                <xsl:text>aggregation-composite-arrow</xsl:text>
                            </xsl:when>
                            <xsl:when test="$start-marker-temp = 'aggregation-aggregate' and
                                            key('xmi.id', $start-id)/@isNavigable='true' and
                                            key('xmi.id', $end-id)/@isNavigable='false'">
                                <xsl:text>aggregation-aggregate-arrow</xsl:text>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select="$start-marker-temp"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="end-marker">
                        <xsl:choose>
                            <xsl:when test="$end-marker-temp = ''">
                                <xsl:if test="key('xmi.id', $start-id)/@isNavigable='false' and
                                              key('xmi.id', $end-id)/@isNavigable='true'">
                                    <xsl:text>arrow</xsl:text>
                                </xsl:if>
                            </xsl:when>
                            <xsl:when test="$start-marker-temp = 'aggregation-composite' and
                                            key('xmi.id', $start-id)/@isNavigable='false' and
                                            key('xmi.id', $end-id)/@isNavigable='true'">
                                <xsl:text>aggregation-composite-arrow</xsl:text>
                            </xsl:when>
                            <xsl:when test="$start-marker-temp = 'aggregation-aggregate' and
                                            key('xmi.id', $start-id)/@isNavigable='false' and
                                            key('xmi.id', $end-id)/@isNavigable='true'">
                                <xsl:text>aggregation-aggregate-arrow</xsl:text>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:value-of select="$end-marker-temp"/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <path d="{$commands}">
                        <xsl:apply-templates select="." mode="style"/>
                        <xsl:variable name="marker-color">
                            <xsl:choose>
                                <xsl:when test="UML:DiagramElement.property/UML:Property[@key='stroke']">
                                    <xsl:call-template name="create-color-id">
                                        <xsl:with-param name="color"
                                            select="UML:DiagramElement.property/UML:Property[@key='stroke']/@value"/>
                                    </xsl:call-template>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:call-template name="create-color-id">
                                        <xsl:with-param name="color" select="'black'"/>
                                    </xsl:call-template>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        <xsl:choose>
                            <xsl:when test="UML:GraphElement.semanticModel//UML:Generalization or
                                            (UML:GraphElement.semanticModel//UML:Abstraction and
                                             UML:GraphElement.contained/UML:GraphNode[@isVisible='true' and
                                             .//UML:SimpleSemanticModelElement[@typeInfo='StereotypeCompartment']])">
                                 <xsl:attribute name="marker-end">url(#generalization-<xsl:value-of select="$marker-color"/>)</xsl:attribute>
                            </xsl:when>
                            <xsl:when test="UML:GraphElement.semanticModel//UML:Dependency or
                                            UML:GraphElement.semanticModel//UML:Transition or
                                            UML:GraphElement.semanticModel//UML:Include or
                                            UML:GraphElement.semanticModel//UML:Extend">
                                 <xsl:attribute name="marker-end">url(#arrow-end-<xsl:value-of select="$marker-color"/>)</xsl:attribute>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:if test="not($start-marker='arrow' and $end-marker='arrow')">
                                    <xsl:if test="$start-marker != ''">
                                        <xsl:attribute name="marker-start">
                                            <xsl:text>url(#</xsl:text>
                                            <xsl:value-of select="$start-marker"/>
                                            <xsl:text>-start-</xsl:text>
                                            <xsl:value-of select="$marker-color"/>
                                            <xsl:text>)</xsl:text>
                                        </xsl:attribute>
                                    </xsl:if>
                                    <xsl:if test="$end-marker != ''">
                                        <xsl:attribute name="marker-end">
                                            <xsl:text>url(#</xsl:text>
                                            <xsl:value-of select="$end-marker"/>
                                            <xsl:text>-end-</xsl:text>
                                            <xsl:value-of select="$marker-color"/>
                                            <xsl:text>)</xsl:text>
                                        </xsl:attribute>
                                    </xsl:if>
                                </xsl:if>
                            </xsl:otherwise>
                        </xsl:choose>
                    </path>
                </xsl:when>
                <xsl:when test="UML:GraphElement.semanticModel//UML:Comment">
                    <path>
                        <xsl:apply-templates select="." mode="style"/>
                        <xsl:attribute name="d">
                            <xsl:text>M 0 0 L </xsl:text>
                            <xsl:value-of select="UML:GraphNode.size/XMI.field[1] - $commentBorder"/>
                            <xsl:text> 0 L </xsl:text>
                            <xsl:value-of select="UML:GraphNode.size/XMI.field[1]"/>
                            <xsl:text> </xsl:text>
                            <xsl:value-of select="$commentBorder"/>
                            <xsl:text> L </xsl:text>
                            <xsl:value-of select="UML:GraphNode.size/XMI.field[1]"/>
                            <xsl:text> </xsl:text>
                            <xsl:value-of select="UML:GraphNode.size/XMI.field[2]"/>
                            <xsl:text> L 0 </xsl:text>
                            <xsl:value-of select="UML:GraphNode.size/XMI.field[2]"/>
                            <xsl:text> z</xsl:text>
                        </xsl:attribute>
                    </path>
                    <path>
                        <xsl:apply-templates select="." mode="style"/>
                        <xsl:attribute name="fill">none</xsl:attribute>
                        <xsl:attribute name="d">
                            <xsl:text>M </xsl:text>
                            <xsl:value-of select="UML:GraphNode.size/XMI.field[1] - $commentBorder"/>
                            <xsl:text> 0 L </xsl:text>
                            <xsl:value-of select="UML:GraphNode.size/XMI.field[1] - $commentBorder"/>
                            <xsl:text> </xsl:text>
                            <xsl:value-of select="$commentBorder"/>
                            <xsl:text> L </xsl:text>
                            <xsl:value-of select="UML:GraphNode.size/XMI.field[1]"/>
                            <xsl:text> </xsl:text>
                            <xsl:value-of select="$commentBorder"/>
                        </xsl:attribute>
                    </path>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:choose>
                        <xsl:when test="UML:GraphNode.size/XMI.field[2] = '1.0'">
                            <line x2="{UML:GraphNode.size/XMI.field[1]}">
                                <xsl:choose>
                                    <xsl:when test="UML:GraphElement.semanticModel/
                                                    UML:SimpleSemanticModelElement[@typeInfo='CompartmentSeparator']">
                                        <xsl:apply-templates select="../.." mode="style"/>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <xsl:apply-templates select="." mode="style"/>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </line>
                        </xsl:when>
                        <xsl:when test="UML:GraphNode.size/XMI.field[1] = '1.0'">
                            <line y2="{UML:GraphNode.size/XMI.field[2]}">
                                <xsl:choose>
                                    <xsl:when test="UML:GraphElement.semanticModel/
                                                    UML:SimpleSemanticModelElement[@typeInfo='CompartmentSeparator']">
                                        <xsl:apply-templates select="../.." mode="style"/>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <xsl:apply-templates select="." mode="style"/>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </line>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:variable name="strokeStyle">
                                <xsl:call-template name="getStrokeStyle"/>
                            </xsl:variable>
                            <xsl:variable name="fillStyle">
                                <xsl:call-template name="getFillStyle"/>
                            </xsl:variable>
                            <xsl:if test="$strokeStyle != 'none' or $fillStyle != 'none'">
                                <rect stroke="{$strokeStyle}" fill="{$fillStyle}">
                                    <xsl:apply-templates select="." mode="style">
                                        <xsl:with-param name="ignore-stroke-and-fill" select="'true'"/>
                                    </xsl:apply-templates>
                                    <xsl:if test="UML:GraphElement.semanticModel//
                                                  UML:CompositeState or
                                                  UML:GraphElement.semanticModel//
                                                  UML:SimpleState">
                                        <xsl:attribute name="rx">7</xsl:attribute>
                                        <xsl:attribute name="ry">7</xsl:attribute>
                                    </xsl:if>
                                    <xsl:apply-templates select="*" mode="attrib"/>
                                </rect>
                            </xsl:if>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:apply-templates select="UML:GraphElement.semanticModel/*" mode="renderText">
                <xsl:with-param name="fontSize">
                    <xsl:choose>
                        <xsl:when test="$fontSizeVar=''">11</xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$fontSizeVar"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:with-param>
                <xsl:with-param name="fontColor">
                    <xsl:choose>
                        <xsl:when test="$fontColorVar=''">black</xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$fontColorVar"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:with-param>
                <xsl:with-param name="fontFamily">
                    <xsl:choose>
                        <xsl:when test="$fontFamilyVar=''">Helvetica</xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$fontFamilyVar"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:with-param>
            </xsl:apply-templates>

            <xsl:if test="$depthVar &lt; $maxDepth">
                <xsl:for-each select="UML:GraphElement.contained/UML:GraphNode[@isVisible='true']|
                                      UML:GraphElement.contained/UML:GraphEdge[@isVisible='true']|
                                      UML:GraphElement.contained/UML:Polyline[@isVisible='true']|
                                      UML:GraphElement.contained/UML:Ellipse[@isVisible='true']">
                    <xsl:call-template name="processGraphNode">
                        <xsl:with-param name="depth" select="$depthVar"/>
                        <xsl:with-param name="fontSize" select="$fontSizeVar"/>
                        <xsl:with-param name="fontColor" select="$fontColorVar"/>
                        <xsl:with-param name="fontFamily" select="$fontFamilyVar"/>
                    </xsl:call-template>
                </xsl:for-each>
            </xsl:if>
        </g>
    </xsl:template>

    
    <!--
      -  Make tspan elements for a text containing line feeds.
      -->
    <xsl:template name="makeSpans">
        <xsl:param name="text"/>
        <xsl:param name="x" select="0"/>
        <xsl:param name="createDY">false</xsl:param>
        <xsl:choose>
            <xsl:when test="contains($text, '&#10;')">
                <xsl:element name="tspan">
                    <xsl:attribute name="x"><xsl:value-of select="$x"/></xsl:attribute>
                    <xsl:if test="$createDY='true'">
                        <xsl:attribute name="dy">1.2em</xsl:attribute>
                    </xsl:if>
                    <xsl:value-of select="substring-before($text, '&#10;')"/>
                </xsl:element>
                <xsl:call-template name="makeSpans">
                    <xsl:with-param name="text"
                        select="substring-after($text, '&#10;')"/>
                    <xsl:with-param name="createDY" select="'true'"/>
                    <xsl:with-param name="x" select="$x"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:element name="tspan">
                    <xsl:attribute name="x"><xsl:value-of select="$x"/></xsl:attribute>
                    <xsl:if test="$createDY='true'">
                        <xsl:attribute name="dy">1.2em</xsl:attribute>
                    </xsl:if>
                    <xsl:value-of select="$text"/>
                </xsl:element>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="*|@*|text()" mode="makeSpans">
        <xsl:call-template name="makeSpans">
            <xsl:with-param name="text" select="."/>
        </xsl:call-template>
    </xsl:template>

    
    <!--
      -  Output the name of a class or interface (by overriding this
      -  template, you can, for example, create links to the outside world).
      -->
    <xsl:template name="outputName">
        <xsl:for-each select="key('xmi.id', @xmi.idref)">
            <xsl:apply-templates select="@name" mode="makeSpans"/>
        </xsl:for-each>
    </xsl:template>


    <!--
      -  Render text of elements.
      -->
    <xsl:template name="getVisibilitySymbol">
        <xsl:param name="visibility"/>
        <xsl:choose>
            <xsl:when test="$visibility='public'">+</xsl:when>
            <xsl:when test="$visibility='protected'">#</xsl:when>
            <xsl:when test="$visibility='package'">~</xsl:when>
            <xsl:when test="$visibility='private'">-</xsl:when>
        </xsl:choose>
    </xsl:template>
    <xsl:template name="getMultiplicity">
        <xsl:param name="lowerBound"/>
        <xsl:param name="upperBound"/>
        <xsl:choose>
            <xsl:when test="$lowerBound = $upperBound">
                <xsl:value-of select="$lowerBound"/>
            </xsl:when>
            <xsl:when test="$lowerBound = '0' and $upperBound = '-1'">*</xsl:when>
            <xsl:when test="$lowerBound != '0' and $upperBound = '-1'">
                <xsl:value-of select="$lowerBound"/>
                <xsl:text>..*</xsl:text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$lowerBound"/>
                <xsl:text>..</xsl:text>
                <xsl:value-of select="$upperBound"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="UML:ExtensionPoint" mode="getName">
        <xsl:apply-templates select="key('xmi.id', @xmi.idref)/@name" mode="makeSpans"/>
    </xsl:template>
    <xsl:template match="UML:ExtensionPoint" mode="getLocation">
        <xsl:apply-templates select="key('xmi.id', @xmi.idref)/@location" mode="makeSpans"/>
    </xsl:template>
    <xsl:template match="UML:Class|UML:AssociationClass" mode="getName">
        <xsl:element name="tspan">
            <xsl:if test="key('xmi.id', @xmi.idref)/@isAbstract='true'">
                <xsl:attribute name="font-style">italic</xsl:attribute>
            </xsl:if>
            <xsl:call-template name="outputName"/>
        </xsl:element>
    </xsl:template>
    <xsl:template match="UML:UseCase" mode="getName">
        <xsl:element name="tspan">
            <xsl:if test="key('xmi.id', @xmi.idref)/@isAbstract='true'">
                <xsl:attribute name="font-style">italic</xsl:attribute>
            </xsl:if>
            <xsl:apply-templates select="key('xmi.id', @xmi.idref)/@name" mode="makeSpans"/>
        </xsl:element>
    </xsl:template>
    <xsl:template match="UML:Interface | UML:DataType" mode="getName">
        <xsl:call-template name="outputName"/>
    </xsl:template>
    <xsl:template match="UML:Stereotype | UML:Package |
                         UML:Attribute | UML:Operation | UML:ObjectFlowState |
                         UML:Parameter | UML:AssociationEnd | UML:Collaboration |
                         UML:ClassifierRole | UML:Actor | UML:SynchState |
                         UML:Association | UML:SimpleState | UML:CompositeState |
                         UML:Pseudostate | UML:FinalState | UML:CallEvent |
                         UML:Transition | UML:Dependency"
                   mode="getName">
        <xsl:apply-templates select="key('xmi.id', @xmi.idref)/@name" mode="makeSpans"/>
    </xsl:template>
    <xsl:template match="UML:Guard" mode="getExpression">
        <xsl:apply-templates select="key('xmi.id', @xmi.idref)/UML:Guard.expression/UML:BooleanExpression/@body"
                               mode="makeSpans"/>
    </xsl:template>
    <xsl:template match="UML:CallAction" mode="getExpression">
        <xsl:apply-templates select="key('xmi.id', @xmi.idref)/UML:Action.script/UML:ActionExpression/@body"
                               mode="makeSpans"/>
    </xsl:template>
    <xsl:template match="UML:CallAction" mode="getLabel">
        <xsl:for-each select="key('xmi.id', @xmi.idref)">
            <xsl:choose>
                <xsl:when test="name(..)='UML:State.entry'">
                    <xsl:text>entry</xsl:text>
                </xsl:when>
                <xsl:when test="name(..)='UML:State.exit'">
                    <xsl:text>exit</xsl:text>
                </xsl:when>
                <xsl:when test="name(..)='UML:State.doActivity'">
                    <xsl:text>do</xsl:text>
                </xsl:when>
            </xsl:choose>
        </xsl:for-each>
    </xsl:template>
    <xsl:template match="UML:Attribute | UML:Operation | UML:AssociationEnd" mode="getVisibility">
        <xsl:call-template name="getVisibilitySymbol">
            <xsl:with-param name="visibility" select="key('xmi.id', @xmi.idref)/@visibility"/>
        </xsl:call-template>
    </xsl:template>
    <xsl:template match="UML:AssociationEnd" mode="getMultiplicity">
        <xsl:call-template name="getMultiplicity">
            <xsl:with-param name="lowerBound"
                            select="string(key('xmi.id', @xmi.idref)/UML:AssociationEnd.multiplicity/
                                    UML:Multiplicity/UML:Multiplicity.range/UML:MultiplicityRange/@lower)"/>
            <xsl:with-param name="upperBound"
                            select="string(key('xmi.id', @xmi.idref)/UML:AssociationEnd.multiplicity/
                                    UML:Multiplicity/UML:Multiplicity.range/UML:MultiplicityRange/@upper)"/>
        </xsl:call-template>
    </xsl:template>
    <xsl:template match="UML:Attribute" mode="getMultiplicity">
        <xsl:call-template name="getMultiplicity">
            <xsl:with-param name="lowerBound"
                            select="string(key('xmi.id', @xmi.idref)/UML:StructuralFeature.multiplicity/
                                    UML:Multiplicity/UML:Multiplicity.range/UML:MultiplicityRange/@lower)"/>
            <xsl:with-param name="upperBound"
                            select="string(key('xmi.id', @xmi.idref)/UML:StructuralFeature.multiplicity/
                                    UML:Multiplicity/UML:Multiplicity.range/UML:MultiplicityRange/@upper)"/>
        </xsl:call-template>
    </xsl:template>
    <xsl:template match="UML:Attribute" mode="getInitialValue">
        <xsl:apply-templates select="key('xmi.id', @xmi.idref)/UML:Attribute.initialValue/UML:Expression/@body"
                               mode="makeSpans"/>
    </xsl:template>
    <xsl:template match="UML:Parameter" mode="getKind">
        <xsl:apply-templates select="key('xmi.id', @xmi.idref)/@kind" mode="makeSpans"/>
    </xsl:template>
    <xsl:template match="*|text()" mode="getName">
        <xsl:apply-templates mode="getName"/>
    </xsl:template>
    <xsl:template match="*|text()" mode="getExpression">
        <xsl:apply-templates mode="getExpression"/>
    </xsl:template>
    <xsl:template match="*|text()" mode="getMultiplicity">
        <xsl:apply-templates mode="getMultiplicity"/>
    </xsl:template>
    <xsl:template match="*|text()" mode="getVisibility">
        <xsl:apply-templates mode="getVisibility"/>
    </xsl:template>
    <xsl:template match="*|text()" mode="getInitialValue">
        <xsl:apply-templates mode="getInitialValue"/>
    </xsl:template>
    <xsl:template match="*|text()" mode="getKind">
        <xsl:apply-templates mode="getKind"/>
    </xsl:template>
    
    <xsl:template match="UML:SimpleSemanticModelElement[@typeInfo!='']" mode="renderText">
        <xsl:param name="fontSize"/>
        <xsl:param name="fontColor"/>
        <xsl:param name="fontFamily"/>
        <xsl:element name="text">
            <xsl:attribute name="y"><xsl:value-of select="$fontSize - 1"/></xsl:attribute>
            <xsl:if test="$fontSize!=$defaultFontSize">
                <xsl:attribute name="font-size">
                    <xsl:value-of select="$fontSize"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:if test="$fontFamily!=$defaultFontFamily">
                <xsl:attribute name="font-family">
                    <xsl:value-of select="$fontFamily"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:if test="$fontColor!='black'">
                <xsl:attribute name="fill">
                    <xsl:value-of select="$fontColor"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:choose>
                <xsl:when test="@typeInfo='Name'">
                    <xsl:if test="(../../../../UML:GraphElement.semanticModel/
                                               UML:SimpleSemanticModelElement
                                               [@typeInfo='NameCompartment'] and
                                   (../../../../../../UML:GraphElement.semanticModel//UML:Class or
                                    ../../../../../../UML:GraphElement.semanticModel//UML:Interface or
                                    ../../../../../../UML:GraphElement.semanticModel//UML:Actor or
                                    ../../../../../../UML:GraphElement.semanticModel/
                                                      UML:SimpleSemanticModelElement/
                                                      @typeInfo='AssociationClassNode')) or
                                  ../../../../UML:GraphElement.semanticModel/*[@presentation='Ball']//UML:Interface">
                        <xsl:attribute name="font-weight">bold</xsl:attribute>
                    </xsl:if>
                    <xsl:if test="../../../../../../UML:GraphElement.semanticModel/
                                              UML:SimpleSemanticModelElement
                                              [@typeInfo='NamespaceCompartment']">
                        <xsl:attribute name="font-style">italic</xsl:attribute>
                    </xsl:if>
                    <xsl:apply-templates select="ancestor::*[
                                                     UML:GraphElement.semanticModel[
                                                         not(UML:SimpleSemanticModelElement)
                                                     ]
                                                 ][1]/UML:GraphElement.semanticModel" mode="getName"/>
                </xsl:when>
                <xsl:when test="@typeInfo='Label'">
                    <xsl:choose>
                        <xsl:when test="../../../../UML:GraphElement.semanticModel/
                                        UML:SimpleSemanticModelElement/
                                        @typeInfo='ExtensionpointCompartment'">
                            <xsl:attribute name="font-weight">bold</xsl:attribute>
                            <xsl:text>Extension Points</xsl:text>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:apply-templates select="ancestor::*[
                                                             UML:GraphElement.semanticModel[
                                                                not(UML:SimpleSemanticModelElement)
                                                             ]
                                                         ][1]/UML:GraphElement.semanticModel" mode="getLabel"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <xsl:when test="@typeInfo='Expression'">
                    <xsl:apply-templates select="ancestor::*[
                                                     UML:GraphElement.semanticModel[
                                                         not(UML:SimpleSemanticModelElement)
                                                     ]
                                                 ][1]/UML:GraphElement.semanticModel" mode="getExpression"/>
                </xsl:when>
                <xsl:when test="@typeInfo='Visibility'">
                    <xsl:apply-templates select="ancestor::UML:GraphNode[
                                                     UML:GraphElement.semanticModel[
                                                         not(UML:SimpleSemanticModelElement)
                                                     ]
                                                 ][1]/UML:GraphElement.semanticModel" mode="getVisibility"/>
                </xsl:when>
                <xsl:when test="@typeInfo='Multiplicity'">
                    <xsl:apply-templates select="ancestor::UML:GraphNode[
                                                     UML:GraphElement.semanticModel[
                                                         not(UML:SimpleSemanticModelElement)
                                                     ]
                                                 ][1]/UML:GraphElement.semanticModel" mode="getMultiplicity"/>
                </xsl:when>
                <xsl:when test="@typeInfo='InitialValue'">
                    <xsl:apply-templates select="ancestor::UML:GraphNode[
                                                     UML:GraphElement.semanticModel[
                                                         not(UML:SimpleSemanticModelElement)
                                                     ]
                                                 ][1]/UML:GraphElement.semanticModel" mode="getInitialValue"/>
                </xsl:when>
                <xsl:when test="@typeInfo='Kind'">
                    <xsl:apply-templates select="ancestor::UML:GraphNode[
                                                     UML:GraphElement.semanticModel[
                                                         not(UML:SimpleSemanticModelElement)
                                                     ]
                                                 ][1]/UML:GraphElement.semanticModel" mode="getKind"/>
                </xsl:when>
                <xsl:when test="@typeInfo='Location'">
                    <xsl:apply-templates select="ancestor::UML:GraphNode[
                                                     UML:GraphElement.semanticModel[
                                                         not(UML:SimpleSemanticModelElement)
                                                     ]
                                                 ][1]/UML:GraphElement.semanticModel" mode="getLocation"/>
                </xsl:when>
                <xsl:when test="@typeInfo='KeywordMetaclass'">
                    <xsl:choose>
                        <xsl:when test="../../../../../../../../UML:GraphElement.semanticModel//UML:Actor">
                            <xsl:text>actor</xsl:text>
                        </xsl:when>
                        <xsl:when test="../../../../../../../../UML:GraphElement.semanticModel//UML:Interface">
                            <xsl:text>interface</xsl:text>
                        </xsl:when>
                        <xsl:when test="../../../../../../UML:GraphElement.semanticModel//UML:Extend">
                            <xsl:text>extend</xsl:text>
                        </xsl:when>
                        <xsl:when test="../../../../../../UML:GraphElement.semanticModel//UML:Include">
                            <xsl:text>include</xsl:text>
                        </xsl:when>
                    </xsl:choose>
                </xsl:when>
                <xsl:when test="@typeInfo='FreeText'">
                    <xsl:apply-templates select="../../UML:GraphElement.contained/UML:TextElement/@text"
                                           mode="makeSpans"/>
                </xsl:when>
                <xsl:when test="@typeInfo='StereotypeStart'">&lt;&lt;</xsl:when>
                <xsl:when test="@typeInfo='StereotypeEnd'">&gt;&gt;</xsl:when>
                <xsl:when test="@typeInfo='NamespaceStart'">
                    <xsl:attribute name="font-style">italic</xsl:attribute>
                    <xsl:text>(from</xsl:text>
                </xsl:when>
                <xsl:when test="@typeInfo='NamespaceSeparator'">
                    <xsl:attribute name="font-style">italic</xsl:attribute>
                    <xsl:text>::</xsl:text>
                </xsl:when>
                <xsl:when test="@typeInfo='NamespaceEnd'">
                    <xsl:attribute name="font-style">italic</xsl:attribute>
                    <xsl:text>)</xsl:text>
                </xsl:when>
                <xsl:when test="@typeInfo='TypeSeparator'">:</xsl:when>
                <xsl:when test="@typeInfo='ParameterStart'">(</xsl:when>
                <xsl:when test="@typeInfo='ParameterEnd'">)</xsl:when>
                <xsl:when test="@typeInfo='ParameterSeparator'">,</xsl:when>
                <xsl:when test="@typeInfo='MultiplicityStart'">[</xsl:when>
                <xsl:when test="@typeInfo='MultiplicityEnd'">]</xsl:when>
                <xsl:when test="@typeInfo='InitialValueSeparator'">=</xsl:when>
                <xsl:when test="@typeInfo='GuardStart'">[</xsl:when>
                <xsl:when test="@typeInfo='GuardEnd'">]</xsl:when>
                <xsl:when test="@typeInfo='EffectStart'">/</xsl:when>
                <xsl:when test="@typeInfo='NameLocationSeparator'">:</xsl:when>
            </xsl:choose>
        </xsl:element>
    </xsl:template>
    <xsl:template match="UML:Comment" mode="renderText">
        <xsl:param name="fontSize"/>
        <xsl:param name="fontColor"/>
        <xsl:param name="fontFamily"/>
        <g transform="{concat('translate(', $commentBorder, ',', $commentBorder, ')')}">
            <xsl:element name="text">
                <xsl:attribute name="y"><xsl:value-of select="$fontSize - 1"/></xsl:attribute>
                <xsl:attribute name="width">
                    <xsl:value-of select="ancestor::UML:GraphNode[1]/UML:GraphNode.size/XMI.field[1] - $commentBorder*2"/>
                </xsl:attribute>
                <xsl:attribute name="height">
                    <xsl:value-of select="ancestor::UML:GraphNode[1]/UML:GraphNode.size/XMI.field[2] - $commentBorder*2"/>
                </xsl:attribute>
                <xsl:if test="$fontSize!=$defaultFontSize">
                    <xsl:attribute name="font-size">
                        <xsl:value-of select="$fontSize"/>
                    </xsl:attribute>
                </xsl:if>
                <xsl:if test="$fontFamily!=$defaultFontFamily">
                    <xsl:attribute name="font-family">
                        <xsl:value-of select="$fontFamily"/>
                    </xsl:attribute>
                </xsl:if>
                <xsl:if test="$fontColor!='black'">
                    <xsl:attribute name="fill">
                        <xsl:value-of select="$fontColor"/>
                    </xsl:attribute>
                </xsl:if>
                <xsl:call-template name="makeSpans">
                    <xsl:with-param name="text" select="key('xmi.id', @xmi.idref)/@body"/>
                </xsl:call-template>
            </xsl:element>
        </g>
    </xsl:template>
    <xsl:template match="*" mode="renderText">
        <xsl:param name="fontSize"/>
        <xsl:param name="fontColor"/>
        <xsl:param name="fontFamily"/>
        <xsl:apply-templates select="*" mode="renderText">
            <xsl:with-param name="fontSize" select="$fontSize"/>
            <xsl:with-param name="fontColor" select="$fontColor"/>
            <xsl:with-param name="fontFamily" select="$fontFamily"/>
        </xsl:apply-templates>
    </xsl:template>


    <!--
      -  Default template (simply apply templates recursively).
      -->
    <xsl:template match="*|@*|text()">
        <xsl:apply-templates/>
    </xsl:template>


    <!--
      -  Default templates for various modes (ignore everything,
      -  don't apply templates recursively).
      -->
    <xsl:template match="*|@*|text()" mode="attrib"/>


    <!--
      -  Root node template (output an empty header if there's no matching diagram).
      -->
    <xsl:template match="XMI">
        <!-- find the diagram to transform -->
        <xsl:variable name="diagramID">
            <xsl:choose>
                <xsl:when test="$diagramName != ''">
                    <xsl:value-of select="XMI.content/UML:Diagram[@name=$diagramName]/@xmi.id"/>
                </xsl:when>
                <xsl:otherwise>
                    <!-- use the first diagram -->
                    <xsl:value-of select="XMI.content/UML:Diagram[1]/@xmi.id"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:choose><xsl:when test="not(XMI.content/UML:Diagram[@xmi.id=$diagramID])">
            <svg width="100" height="10">
            </svg>
        </xsl:when><xsl:otherwise>
            <xsl:apply-templates select="XMI.content/UML:Diagram[@xmi.id=$diagramID]"/>
        </xsl:otherwise></xsl:choose>
    </xsl:template>


    <!--
      -  Template for graph element position.
      -->
    <!--xsl:template match="UML:GraphElement.position" mode="attrib">
        <xsl:attribute name="x"><xsl:value-of select="XMI.field[1]"/></xsl:attribute>
        <xsl:attribute name="y"><xsl:value-of select="XMI.field[2]"/></xsl:attribute>
    </xsl:template-->

    
    <!--
      -  Template for graph node size.
      -->
    <xsl:template match="UML:GraphNode.size" mode="attrib">
        <xsl:attribute name="width"><xsl:value-of select="XMI.field[1]"/></xsl:attribute>
        <xsl:attribute name="height">
            <xsl:choose>
                <xsl:when test="XMI.field[2]='0.0'">0</xsl:when>
                <xsl:otherwise><xsl:value-of select="XMI.field[2]"/></xsl:otherwise>
            </xsl:choose>
        </xsl:attribute>
    </xsl:template>

</xsl:stylesheet>

